# Generated by Django 3.0.2 on 2020-02-25 07:18

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0003_document_version'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='document',
            name='celery_task_id',
        ),
    ]
