# Generated by Django 3.0.10 on 2020-10-02 18:14

from django.db import migrations
import papermerge.core.models.tags


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0024_auto_20200923_1412'),
    ]

    operations = [
        migrations.AlterField(
            model_name='automate',
            name='tags',
            field=papermerge.core.models.tags.UserTaggableManager(blank=True, help_text='A comma-separated list of tags.', through='core.ColoredTag', to='core.Tag', verbose_name='Tags'),
        ),
    ]
