{% extends "admin/base.html" %}

{% load i18n %}

{% block head_title %}{% trans "Preferences" %}{% endblock %}

{% block main_col %}col-12 col-lg-12 col-sm-12 main-col{% endblock %}

 {% block widgetsbar_wrapper %}
 <!-- widgetsbar column N/A here -->
 {% endblock %}

{% block breadcrumb %}
    <div class="col-sm-12 pl-0">
      <a href="{% url 'admin:preferences' %}">
        <i class="fa fa-cogs mx-2"></i>{% trans "Preferences" %}
      </a>
    </div><!-- /.col -->
{% endblock %}

{% block content %}
<div class="preferences">
    <div class="row">
        {% for section in sections %}
          <div class="card">
              <div class="card-body text-center">
                <a href="{% url 'admin:preferences_section' section.name %}">
                    <i class="fa fa-{{section.icon_name}}"></i>
                      {{section}}
                </a>
                <p class="m-2 font-weight-light">{{section.help_text}}</p>
              </div>
          </div>
        {% endfor %}
    </div>
</div>
{% endblock %}