{% load static i18n  %}
<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="utf-8">
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <title>Papermerge | {% trans "Log in" %}</title>

  <link rel="shortcut icon" type="image/png" href="{% static 'admin/img/favicon.ico' %}"/>
  <link rel="stylesheet" href="{% static 'admin/css/all.min.css' %}">
  <!-- icheck bootstrap -->
  <link rel="stylesheet" href="{% static 'admin/css/icheck-bootstrap.min.css' %}">
  <!-- Theme style -->
  <link rel="stylesheet" href="{% static 'admin/css/adminlte.min.css' %}">
  <!-- Google Font: Source Sans Pro -->
  <link href="https://fonts.googleapis.com/css?family=Source+Sans+Pro:300,400,400i,700" rel="stylesheet">
</head>
<body class="hold-transition login-page m-3">
  {% include 'account/_messages.html' %}
  </div>
  {% block content %}
  {% endblock %}
<!-- jQuery -->
<script src="{% static 'admin/js/jquery.min.js' %}"></script>
<!-- Bootstrap 4 -->
<script src="{% static 'admin/js/bootstrap.bundle.min.js' %}"></script>
<!-- AdminLTE App -->
<script src="{% static 'admin/js/adminlte.js' %}"></script>

</body>
</html>