# Generated by Django 3.0.6 on 2020-06-05 10:42

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0013_auto_20200603_0724'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='kvstorepage',
            name='page_id',
        ),
        migrations.AddField(
            model_name='kvstorepage',
            name='page',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='kvstore', to='core.Page'),
        ),
    ]
