# Generated by Django 3.0.6 on 2020-06-03 06:07

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0011_auto_20200602_1532'),
    ]

    operations = [
        migrations.AddField(
            model_name='kvstore',
            name='kv_inherited',
            field=models.BooleanField(default=False),
        ),
        migrations.CreateModel(
            name='KVStoreCompNode',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('node', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='kvstorecomp', to='core.BaseTreeNode')),
            ],
        ),
        migrations.CreateModel(
            name='KVStoreCompItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('comp_item', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='kvstore', to='core.KVStore')),
            ],
        ),
    ]
