# Generated by Django 3.0.2 on 2020-02-18 11:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0002_auto_20200116_1050'),
    ]

    operations = [
        migrations.AddField(
            model_name='document',
            name='version',
            field=models.IntegerField(blank=True, default=0, null=True),
        ),
    ]
