{% load i18n  %}

<!-- To the right -->
<div class="float-right d-none d-sm-inline">
Version {{papermerge_version}}
</div>
<!-- Default to the left -->
<strong>Copyright &copy; 2020 <a href="https://papermerge.com">Papermerge DMS</a>.</strong> {% trans "All rights reserved" %}.