{% extends "account/base.html" %}

{% load i18n %}

{% block head_title %}{% trans "Sign Out" %}{% endblock %}

{% block content %}
    <h1>{% trans "Sign Out" %}</h1>

    <p>{% trans 'Are you sure you want to sign out?' %}</p>

    <form method="post" action="{% url 'account_logout' %}">
      {% csrf_token %}
      {% if redirect_field_value %}
      <input type="hidden" name="{{ redirect_field_name }}" value="{{ redirect_field_value }}"/>
      {% endif %}
        <div class="d-flex">
            <button type="submit" class="btn btn-success m-3">{% trans 'Yes, sign out' %}</button>
            <a href="{% url 'admin:index' %}" class="btn btn-light m-3">{% trans "Nope, take me back!" %}</a>
        </div>
    </form>
{% endblock %}
