{% extends "admin/object_form.html" %}

{% load i18n admin_tags %}

{% block widgetsbar %}
<div class="card">
  <div class="card-body">
    <ul class="collection">
      <li class="collection-item d-flex flex-row-reverse mt-3">
          <a href="{% url 'admin:logs' %}" class="btn btn-primary btn-flat btn-block">
            <i class="fa fa-check"></i>
            {% trans 'Close' %}
          </a>
      </li>
    </ul>
  </div>
</div>
{% endblock %}