# Generated by Django 3.0.10 on 2020-10-18 08:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0026_auto_20201113_1427'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tag',
            name='name',
            field=models.CharField(max_length=100, verbose_name='name'),
        ),
        migrations.AlterUniqueTogether(
            name='tag',
            unique_together={('name', 'user')},
        ),
    ]
