{% extends "account/base.html" %}
{% load i18n %}

{% block content %}
<div class="login-box">
  <!-- /.login-logo -->
  <div class="card">
    <div class="card-body login-card-body">
      <p class="login-box-msg">{% trans "You forgot your password? Here you can easily retrieve a new password." %}</p>
      <form action="{% url 'account_reset_password' %}" method="post">
        {% csrf_token %}
        <div class="input-group mb-3">
          <input type="email" name="email" class="form-control" placeholder="{% trans "Email address" %}">
          <div class="input-group-append">
            <div class="input-group-text">
              <span class="fas fa-envelope"></span>
            </div>
          </div>
        </div>
        <div class="row">
          <div class="col-12">
            <button type="submit" class="btn btn-success btn-block">{% trans "Request new password" %}</button>
          </div>
          <!-- /.col -->
        </div>
      </form>

      <p class="mt-3 mb-1">
        <a href="{{ login_url }}">{% trans "Login" %}</a>
      </p>
    </div>
    <!-- /.login-card-body -->
  </div>
</div>
<!-- /.login-box -->

{% endblock %}