{% load i18n  %}

<form action="{% url 'core:clipboard' %}" method="GET" id="clipboard_form">
    {% csrf_token %}
</form>

<div id="new-folder-modal" class="modal hidden" tabindex="-1" role="dialog">
    <!-- This part is rendered by backbone templates -->
</div>

<div id="rename-modal" class="modal hidden" tabindex="-1" role="dialog">
    <!-- This part is rendered by backbone templates -->
</div>

<div id="tags-modal" class="modal hidden" tabindex="-1" role="dialog">
    <!-- This part is rendered by backbone templates -->
</div>

<div id="access-modal" class="modal hidden" action="" role="dialog">
</div>

<div id="permission-editor-modal" class="modal hidden" role="dialog">
</div>

<div id="message-modal" class="modal" role="dialog">
</div>
