# Generated by Django 3.0.10 on 2020-09-21 04:56

from django.db import migrations, models
import taggit.managers


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0022_auto_20200909_0826'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='tag',
            options={'ordering': ['name'], 'verbose_name': 'Tag', 'verbose_name_plural': 'Tags'},
        ),
        migrations.RemoveField(
            model_name='automate',
            name='plugin_name',
        ),
        migrations.AddField(
            model_name='automate',
            name='tags',
            field=taggit.managers.TaggableManager(help_text='A comma-separated list of tags.', through='core.ColoredTag', to='core.Tag', verbose_name='Tags'),
        ),
        migrations.AlterField(
            model_name='tag',
            name='pinned',
            field=models.BooleanField(default=False, help_text='Pinned tag will be displayed under Documents menu. It serves as shortcut to quickly filter folders/documents associated with this tag'),
        ),
    ]
