

class SidebarPart:

    """
    Wrapper class for managing/rendering document parts
    on sidebar.
    """

    def __init__(self, document):
        # papermerge.core.models.document instance
        self.document = document

    def to_json(self):
        pass
