{% extends "account/base.html" %}

{% load i18n static %}


{% block content %}

<div class="login-box">
  <!-- /.login-logo -->
  <div class="card">
    <div class="card-body login-card-body">
      <p class="login-box-msg">{% trans "For this setup signup is disabled" %}</p>
      <p class="mb-1">
        <a href="{% url 'account_login' %}">{% trans "Back to login" %}</a>
      </p>
    </div>
    <!-- /.login-card-body -->
  </div>
</div> <!-- /.login-box -->

{% endblock %}