{% extends "admin/object_list.html" %}

{% load i18n admin_tags %}

{% block main_buttons %}
{% endblock main_buttons %}

{% block tbody %}
  {% for log in object_list %}
    <tr>
      <td>
        <span class="mr-2">
          <input type="checkbox" name="_selected_action" value="{{ log.id }}" /></span><a href="{% url 'admin:log-update' log.pk %}">{%localized_datetime log.action_time %}</a>
      </td>
      <td>
        <span class="mr-2">{{ log.message | safe | truncatechars:64}}
      </td>
      <td>
        <span class="mr-2"><a href="{% url 'admin:user-update' log.user.id %}">{{ log.user.username}}</a>
      </td>
      <td>
        <span class="mr-2">{{ log.level | log_level}}
      </td>
    </tr>
  {% endfor %}
{% endblock tbody %}
