{% extends "admin/object_list.html" %}

{% block tbody %}
    {% for obj in object_list %}
      <tr>
        <td>
            <span class="mr-2">
                <input type="checkbox" name="_selected_action" value="{{ obj.pk }}" />
            </span>
            <a href="{% url 'admin:group-update' obj.pk %}">{{ obj.name}}</a>
        </td>
      </tr>
  {% endfor %}
{% endblock tbody %}
