# Generated by Django 3.1.5 on 2021-01-18 06:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0036_auto_20210117_1449'),
    ]

    operations = [
        migrations.AddField(
            model_name='page',
            name='hocr_step_0',
            field=models.TextField(default=''),
        ),
        migrations.AddField(
            model_name='page',
            name='hocr_step_1',
            field=models.TextField(default=''),
        ),
        migrations.AddField(
            model_name='page',
            name='hocr_step_2',
            field=models.TextField(default=''),
        ),
        migrations.AddField(
            model_name='page',
            name='hocr_step_3',
            field=models.TextField(default=''),
        ),
    ]
