{% extends "admin/object_list.html" %}

{% load admin_tags %}

{% block breadcrumb %}
    <div class="col-sm-12 pl-0">
      <a href="{% url 'admin:tokens' %}">
        <i class="fa fa-user-secret mx-2"></i>{{ title }}
      </a>
    </div><!-- /.col -->
{% endblock %}

{% block tbody %}
  <tbody>
    {% for token in object_list %}
      <tr>
        <td><span class="mr-2"><input type="checkbox" name="_selected_action" value="{{ token.digest }}" /></span>{{ token.digest|truncatechars:24 }}</td>
        <td>{% localized_datetime token.created %}</td>
        <td>{% localized_datetime token.expiry %}</td>
      </tr>
    {% endfor %}
  </tbody>
{% endblock %}