{% extends "account/base.html" %}

{% load i18n static bootstrap4 %}

{% block content %}
<div class="login-logo">
  <div>
    <img src="{% static 'admin/img/logo_120x116_wbg.png' %}"/>
  </div>
</div>
<div class="login-box">
  <!-- /.login-logo -->
  <div class="card">
    <div class="card-body login-card-body">
      <p class="login-box-msg">{% trans "Sign in to start your session" %}</p>
       <form action="{% url 'account_login' %}" method="post">
      	{% csrf_token %}
        {% bootstrap_form form %}
        <div class="row">
          <div class="col-12">
            <button type="submit" class="btn btn-success btn-block">{% trans "Sign In" %}</button>
          </div>
          <div class="col-12">
            <div class="icheck-primary">
              <input type="checkbox" id="remember" name="remember">
              <label class="font-weight-normal my-2" for="remember">
                {% trans "Remember Me" %}
              </label>
            </div>
          </div>
        </div>
      </form>
      <p class="mb-1">
        <a href="{% url 'account_reset_password' %}">{% trans "I forgot my password" %}</a>
      </p>
    </div>
    <!-- /.login-card-body -->
  </div>
</div> <!-- /.login-box -->

{% endblock %}