# Generated by Django 3.0.6 on 2020-06-01 18:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0007_auto_20200419_0746'),
    ]

    operations = [
        #migrations.RemoveField(
        #    model_name='page',
        #    name='text_deu',
        #),
        #migrations.RemoveField(
        #    model_name='page',
        #    name='text_eng',
        #),
        #migrations.RemoveField(
        #    model_name='page',
        #    name='text_fts',
        #),
        migrations.AddField(
            model_name='page',
            name='norm_breadcrump',
            field=models.CharField(default='', max_length=1024),
        ),
        migrations.AddField(
            model_name='page',
            name='norm_doc_title',
            field=models.CharField(default='', max_length=200),
        ),
        migrations.AddField(
            model_name='page',
            name='norm_folder_title',
            field=models.CharField(default='', max_length=200),
        ),
        migrations.AddField(
            model_name='page',
            name='norm_text',
            field=models.TextField(default=''),
        ),
    ]
