# Generated by Django 3.0.2 on 2020-04-19 06:39

# import django.contrib.postgres.search
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0004_remove_document_celery_task_id'),
    ]

    operations = [
#        migrations.AddField(
#            model_name='basetreenode',
#            name='ancestors_fts',
#            field=django.contrib.postgres.search.SearchVectorField(null=True),
#        ),
#        migrations.AddField(
#            model_name='basetreenode',
#            name='title_fts',
#            field=django.contrib.postgres.search.SearchVectorField(null=True),
#        ),
#        migrations.AddField(
#            model_name='document',
#            name='text_fts',
#            field=django.contrib.postgres.search.SearchVectorField(null=True),
#        ),
#        migrations.AddField(
#            model_name='page',
#            name='text_fts',
#            field=django.contrib.postgres.search.SearchVectorField(null=True),
#        ),
    ]
