# Generated by Django 3.0.8 on 2020-08-28 08:47

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0018_auto_20200721_1320'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='page',
            options={'ordering': ['number']},
        ),
        migrations.AlterField(
            model_name='automate',
            name='dst_folder',
            field=models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, to='core.Folder', verbose_name='Destination Folder'),
        ),
        migrations.AlterField(
            model_name='automate',
            name='extract_page',
            field=models.BooleanField(default=False, verbose_name='Extract Page'),
        ),
        migrations.AlterField(
            model_name='automate',
            name='is_case_sensitive',
            field=models.BooleanField(default=True, verbose_name='Is case sensitive'),
        ),
        migrations.AlterField(
            model_name='automate',
            name='match',
            field=models.CharField(blank=True, max_length=256, verbose_name='Match'),
        ),
        migrations.AlterField(
            model_name='automate',
            name='matching_algorithm',
            field=models.PositiveIntegerField(choices=[(1, 'Any'), (2, 'All'), (3, 'Literal'), (4, 'Regular Expression')], default=1, verbose_name='Matching Algorithm'),
        ),
        migrations.AlterField(
            model_name='automate',
            name='name',
            field=models.CharField(max_length=128, unique=True, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='automate',
            name='plugin_name',
            field=models.CharField(blank=True, choices=[], default=None, max_length=256, null=True, verbose_name='Plugin Name'),
        ),
    ]
