# Generated by Django 3.0.2 on 2020-04-19 07:46

from django.db import migrations


def insert_language_maps(apps, schema_editor):
    pass


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0006_languagemap'),
    ]

    operations = [
        migrations.RunPython(insert_language_maps),
    ]
