# Generated by Django 3.0.2 on 2020-04-19 07:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0005_auto_20200419_0639'),
    ]

    operations = [
    ]
